var gfilter = (function() {


	var type = "canvas";
	var name = "filters";
	var author = "zhigang";
	var getInfo = function() {
		return author + ' ' + type + ' ' + name;
	};
	//网状
	var colorNetProcess = function(binaryData, l) {
		for (var i = 0; i < l; i += 36) {
			binaryData[i + 3] = 0;
			binaryData[i + 7] = 0;
			binaryData[i + 11] = 0;
			binaryData[i + 15] = 0;
		}
	};

	//颜色互换
	var colorExchangeProcess = function(binaryData, l) {
		for (var i = 0; i < l; i += 4) {
			var r = binaryData[i];
			var g = binaryData[i + 1];
			var b = binaryData[i + 2];

			binaryData[i] = g;
			binaryData[i + 1] = b;
			binaryData[i + 2] = r;
		}
	};

	//红绿色盲
	var colorBlindProcess = function(binaryData, l) {
		for (var i = 0; i < l; i += 4) {
			var b = binaryData[i + 2];
			binaryData[i + 2] = b > 127 ? 255 : 0;
		}
	};

	//百叶窗
	var shuttersProcess = function(binaryData, l, w) {
		for (var i = 0; i < l; i += 4) {
			if (i % (w * 8) == 0) {
				i += w * 8;
				continue;
			}
			binaryData[i + 3] = 0;
		}
	};

	//中国剪纸
	var paperCutProcess = function(binaryData, l) {
		for (var i = 0; i < l; i += 4) {
			var r = binaryData[i];
			var g = binaryData[i + 1];
			var b = binaryData[i + 2];

			if (r + g + b >= 383) {
				binaryData[i] = 0;
				binaryData[i + 1] = 0;
				binaryData[i + 2] = 0;
				binaryData[i + 3] = 255;
			} else {
				binaryData[i] = 255;
				binaryData[i + 1] = 255;
				binaryData[i + 2] = 255;
				binaryData[i + 3] = 0;
			}
		}
	};

	/**
	 * invert color value of pixel, new pixel = RGB(255-r, 255-g, 255 - b)
	 *
	 * @param binaryData - canvas's imagedata.data
	 * @param l - length of data (width * height of image data)
	 */
	var colorInvertProcess = function(binaryData, l) {
		for (var i = 0; i < l; i += 4) {
			var r = binaryData[i];
			var g = binaryData[i + 1];
			var b = binaryData[i + 2];

			binaryData[i] = 255 - r;
			binaryData[i + 1] = 255 - g;
			binaryData[i + 2] = 255 - b;
		}
	};

	/**
	 * adjust color values and make it more darker and gray...
	 *
	 * @param binaryData
	 * @param l
	 */
	var colorAdjustProcess = function(binaryData, l) {
		for (var i = 0; i < l; i += 4) {
			var r = binaryData[i];
			var g = binaryData[i + 1];
			var b = binaryData[i + 2];

			binaryData[i] = (r * 0.272) + (g * 0.534) + (b * 0.131);
			binaryData[i + 1] = (r * 0.349) + (g * 0.686) + (b * 0.168);
			binaryData[i + 2] = (r * 0.393) + (g * 0.769) + (b * 0.189);
		}
	};

	/**
	 * deep clone image data of canvas
	 *
	 * @param context
	 * @param src
	 * @returns
	 */
	var copyImageData = function(context, src) {
		var dst = context.createImageData(src.width, src.height);
		dst.data.set(src.data);
		return dst;
	};

	/**
	 * convolution - keneral size 5*5 - blur effect filter(模糊效果)
	 *
	 * @param context
	 * @param canvasData
	 */
	var blurProcess = function(context, canvasData) {
			console.log("Canvas Filter - blur process");
			var tempCanvasData = copyImageData(context, canvasData);
			var sumred = 0.0,
				sumgreen = 0.0,
				sumblue = 0.0;
			for (var x = 0; x < tempCanvasData.width; x++) {
				for (var y = 0; y < tempCanvasData.height; y++) {

					// Index of the pixel in the array    
					var idx = (x + y * tempCanvasData.width) * 4;
					for (var subCol = -2; subCol <= 2; subCol++) {
						var colOff = subCol + x;
						if (colOff < 0 || colOff >= tempCanvasData.width) {
							colOff = 0;
						}
						for (var subRow = -2; subRow <= 2; subRow++) {
							var rowOff = subRow + y;
							if (rowOff < 0 || rowOff >= tempCanvasData.height) {
								rowOff = 0;
							}
							var idx2 = (colOff + rowOff * tempCanvasData.width) * 4;
							var r = tempCanvasData.data[idx2 + 0];
							var g = tempCanvasData.data[idx2 + 1];
							var b = tempCanvasData.data[idx2 + 2];
							sumred += r;
							sumgreen += g;
							sumblue += b;
						}
					}

					// calculate new RGB value
					var nr = (sumred / 25.0);
					var ng = (sumgreen / 25.0);
					var nb = (sumblue / 25.0);

					// clear previous for next pixel point
					sumred = 0.0;
					sumgreen = 0.0;
					sumblue = 0.0;

					// assign new pixel value    
					canvasData.data[idx + 0] = nr; // Red channel    
					canvasData.data[idx + 1] = ng; // Green channel    
					canvasData.data[idx + 2] = nb; // Blue channel    
					canvasData.data[idx + 3] = 255; // Alpha channel    
				}
			}
	};

		/**
		 * after pixel value - before pixel value + 128
		 * 浮雕效果
		 */
		var reliefProcess = function(context, canvasData) {
			console.log("Canvas Filter - relief process");
			var tempCanvasData = copyImageData(context, canvasData);
			for (var x = 1; x < tempCanvasData.width - 1; x++) {
				for (var y = 1; y < tempCanvasData.height - 1; y++) {

					// Index of the pixel in the array    
					var idx = (x + y * tempCanvasData.width) * 4;
					var bidx = ((x - 1) + y * tempCanvasData.width) * 4;
					var aidx = ((x + 1) + y * tempCanvasData.width) * 4;

					// calculate new RGB value
					var nr = tempCanvasData.data[aidx + 0] - tempCanvasData.data[bidx + 0] + 128;
					var ng = tempCanvasData.data[aidx + 1] - tempCanvasData.data[bidx + 1] + 128;
					var nb = tempCanvasData.data[aidx + 2] - tempCanvasData.data[bidx + 2] + 128;
					nr = (nr < 0) ? 0 : ((nr > 255) ? 255 : nr);
					ng = (ng < 0) ? 0 : ((ng > 255) ? 255 : ng);
					nb = (nb < 0) ? 0 : ((nb > 255) ? 255 : nb);

					// assign new pixel value    
					canvasData.data[idx + 0] = nr; // Red channel    
					canvasData.data[idx + 1] = ng; // Green channel    
					canvasData.data[idx + 2] = nb; // Blue channel    
					canvasData.data[idx + 3] = 255; // Alpha channel    
				}
			}
		};

	/**
	 * 	before pixel value - after pixel value + 128
	 *  雕刻效果
	 *
	 * @param canvasData
	 */
	var diaokeProcess = function(context, canvasData) {
		console.log("Canvas Filter - process");
		var tempCanvasData = copyImageData(context, canvasData);
		for (var x = 1; x < tempCanvasData.width - 1; x++) {
			for (var y = 1; y < tempCanvasData.height - 1; y++) {

				// Index of the pixel in the array    
				var idx = (x + y * tempCanvasData.width) * 4;
				var bidx = ((x - 1) + y * tempCanvasData.width) * 4;
				var aidx = ((x + 1) + y * tempCanvasData.width) * 4;

				// calculate new RGB value
				var nr = tempCanvasData.data[bidx + 0] - tempCanvasData.data[aidx + 0] + 128;
				var ng = tempCanvasData.data[bidx + 1] - tempCanvasData.data[aidx + 1] + 128;
				var nb = tempCanvasData.data[bidx + 2] - tempCanvasData.data[aidx + 2] + 128;
				nr = (nr < 0) ? 0 : ((nr > 255) ? 255 : nr);
				ng = (ng < 0) ? 0 : ((ng > 255) ? 255 : ng);
				nb = (nb < 0) ? 0 : ((nb > 255) ? 255 : nb);

				// assign new pixel value    
				canvasData.data[idx + 0] = nr; // Red channel    
				canvasData.data[idx + 1] = ng; // Green channel    
				canvasData.data[idx + 2] = nb; // Blue channel    
				canvasData.data[idx + 3] = 255; // Alpha channel    
			}
		}
	};

	/**
	 * mirror reflect
	 *
	 * @param context
	 * @param canvasData
	 */
	var mirrorProcess = function(context, canvasData) {
		console.log("Canvas Filter - process");
		var tempCanvasData = copyImageData(context, canvasData);
		for (var x = 0; x < tempCanvasData.width; x++) // column
		{
			for (var y = 0; y < tempCanvasData.height; y++) // row
			{

				// Index of the pixel in the array    
				var idx = (x + y * tempCanvasData.width) * 4;
				var midx = (((tempCanvasData.width - 1) - x) + y * tempCanvasData.width) * 4;

				// assign new pixel value    
				canvasData.data[midx + 0] = tempCanvasData.data[idx + 0]; // Red channel    
				canvasData.data[midx + 1] = tempCanvasData.data[idx + 1];; // Green channel    
				canvasData.data[midx + 2] = tempCanvasData.data[idx + 2];; // Blue channel    
				canvasData.data[midx + 3] = 255; // Alpha channel    
			}
		}
	};

	return {
		//some filter function
		//反色
		invertColor: function(myCanvas, context) {
			var len = myCanvas.width * myCanvas.height * 4;
			var canvasData = context.getImageData(0, 0, myCanvas.width, myCanvas.height);
			var binaryData = canvasData.data;

			// Processing all the pixels
			colorInvertProcess(binaryData, len);
			// Copying back canvas data to canvas
			context.putImageData(canvasData, 0, 0);
		},
		//灰色调

		adjustColor: function(myCanvas, context) {
			var len = myCanvas.width * myCanvas.height * 4;
			var canvasData = context.getImageData(0, 0, myCanvas.width, myCanvas.height);
			var binaryData = canvasData.data;

			// Processing all the pixels
			colorAdjustProcess(binaryData, len);

			// Copying back canvas data to canvas
			context.putImageData(canvasData, 0, 0);
		},
		//模糊

		blurImage: function(myCanvas, context) {
			var len = myCanvas.width * myCanvas.height * 4;
			var canvasData = context.getImageData(0, 0, myCanvas.width, myCanvas.height);

			// Processing all the pixels
			blurProcess(context, canvasData);

			// Copying back canvas data to canvas
			context.putImageData(canvasData, 0, 0);
		},

		//浮雕
		fudiaoImage: function(myCanvas, context) {
			var len = myCanvas.width * myCanvas.height * 4;
			var canvasData = context.getImageData(0, 0, myCanvas.width, myCanvas.height);

			// Processing all the pixels
			reliefProcess(context, canvasData);

			// Copying back canvas data to canvas
			context.putImageData(canvasData, 0, 0);
		},

		//刻雕
		kediaoImage: function(myCanvas, context) {
			var len = myCanvas.width * myCanvas.height * 4;
			var canvasData = context.getImageData(0, 0, myCanvas.width, myCanvas.height);

			// Processing all the pixels
			diaokeProcess(context, canvasData);

			// Copying back canvas data to canvas
			context.putImageData(canvasData, 0, 0);
		},

		//Image in mirror镜像
		mirrorImage: function(myCanvas, context) {
			var len = myCanvas.width * myCanvas.height * 4;
			var canvasData = context.getImageData(0, 0, myCanvas.width, myCanvas.height);

			// Processing all the pixels
			mirrorProcess(context, canvasData);

			// Copying back canvas data to canvas
			context.putImageData(canvasData, 0, 0);
		},

		//剪纸
		ChinaPaperCut: function(myCanvas, context) {
			var len = myCanvas.width * myCanvas.height * 4;
			var canvasData = context.getImageData(0, 0, myCanvas.width, myCanvas.height);
			var binaryData = canvasData.data;

			// Processing all the pixels
			paperCutProcess(binaryData, len);

			// Copying back canvas data to canvas
			context.putImageData(canvasData, 0, 0);
		},

		//百叶窗
		shutters: function(myCanvas, context) {
			var len = myCanvas.width * myCanvas.height * 4;
			var canvasData = context.getImageData(0, 0, myCanvas.width, myCanvas.height);
			var binaryData = canvasData.data;

			// Processing all the pixels
			shuttersProcess(binaryData, len, myCanvas.width);

			// Copying back canvas data to canvas
			context.putImageData(canvasData, 0, 0);
		},

		//红绿色盲
		colorBlind: function(myCanvas, context) {

			var len = myCanvas.width * myCanvas.height * 4;
			var canvasData = context.getImageData(0, 0, myCanvas.width, myCanvas.height);
			var binaryData = canvasData.data;

			// Processing all the pixels
			colorBlindProcess(binaryData, len);

			// Copying back canvas data to canvas
			context.putImageData(canvasData, 0, 0);
		},
		//颜色互换

		colorExchange: function(myCanvas, context) {

			var len = myCanvas.width * myCanvas.height * 4;
			var canvasData = context.getImageData(0, 0, myCanvas.width, myCanvas.height);
			var binaryData = canvasData.data;

			// Processing all the pixels
			colorExchangeProcess(binaryData, len);

			// Copying back canvas data to canvas
			context.putImageData(canvasData, 0, 0);
		},
		//网状

		colorNet: function(myCanvas, context) {

			var len = myCanvas.width * myCanvas.height * 4;
			var canvasData = context.getImageData(0, 0, myCanvas.width, myCanvas.height);
			var binaryData = canvasData.data;

			// Processing all the pixels
			colorNetProcess(binaryData, len);

			// Copying back canvas data to canvas
			context.putImageData(canvasData, 0, 0);
		},
	};
})();