/**
 * 这里来专门存储一些工具类的指令
 */
rootModule
    .filter('videoTime', function($filter) {
        return function(value) {
            value = parseInt(value);
            var min = parseInt(value / 60);
            var second = parseInt(value % 60);
            return (min < 10 ? '0' + min : min) + ':' + (second < 10 ? '0' + second : second);
        }
    })
    .filter('to_trusted',function($sce) {
            return function(text) {
                return $sce.trustAsHtml(text);
            }
        })
    .filter('decode', function($filter, $sce) {
        return function(value) {
            try {
                value = decodeURIComponent(value);
                value = $sce.trustAsHtml(value);
            } catch (e) {

            }
            return value;
        }
    })
    .directive("scanner",function(barCodeScannerUtil,ENV){
        var linker = function(scope, element, attrs) {
            element.bind('click',function(){
                 //调用新版的自定义相机界面
                 if(typeof HSPlugin!='undefined'){
                    HSPlugin.execFunc("startCapture",[ENV.posterScannerServer,ENV.videoScannerServer],function(result){
                        barCodeScannerUtil.doScannerResult(result);
                        console.log('scanner result:'+result);
                    },function(err){
                        console.log('出错了！')
                    })
                 }
            })
        }
        return {
            restrict: "A",
            link: linker,
            replace: true
        };
    })
    /**
     * 提供界面元素图片大小计算服务
     * @param  {[type]} ){                 } [description]
     * @return {[type]}     [description]
     */
    .service('ImageSvc', function(){
        var defaultImg="img/black.png"
        /**
         * 计算图片显示的大小
         * @param  {[type]} scope     页面作用域
         * @param  {[type]} itemCount 需要显示的数量
         * @param  {[type]} itemRate  显示的比例，4:3
         * @param  {[type]} float     剩余多少距离，如首页不是均匀显示3张而是3张半
         * @return {[type]}           [description]
         */
        function calcItem(scope,itemCount,itemRate,float){
            //这里来计算页面上乐配图片应该被显示的大小
            //1.获取屏幕的宽度
            var windowWidth=window.innerWidth;
            //2.计算单个项目的宽度,默认显示两张
            var itemWidth=windowWidth/(itemCount+float);
            //3.根据图片比例计算单个项目的高度，默认4:3
            var itemHeight=itemWidth/itemRate;
            scope.itemWidth=itemWidth; //设置到当前作用域
            scope.itemHeight=itemHeight; //设置到当前作用域
            return {
                itemWidth:itemWidth,
                itemHeight:itemHeight
            }
        }
        /**
         * 创建缩略图
         * @param  {[type]} originUrl [description]
         * @param  {[type]} type      [description]
         * @return {[type]}           [description]
         */
        function createImage(originUrl,type){
            if(originUrl&&originUrl!=""){
                var items=originUrl.split("/");
                items[items.length-1]=type+"/"+items[items.length-1];
                return items.join('/');
            }else{
                return defaultImg;
            }
        }
        return {
            /**
             * 计算图片宽度函数
             * @type {[type]}
             */
            calcItem:calcItem,
            /**
             * 获取用户的头像96X96
             * @param  {[type]} originUrl 原始图片地址
             * @return {[type]}           图片地址
             */
            getUserHead_96:function(originUrl){
                return createImage(originUrl,1);
            },
            /**
             * 获取乐配缩略图
             * @param  {[type]} originUrl 原始图片地址
             * @return {[type]}           图片地址
             */
            getPlayThumb:function(originUrl){
                return createImage(originUrl,4);
            },
            /**
             * 获取海报缩略图
             * @param  {[type]} originUrl 原始图片地址
             * @return {[type]}           图片地址
             */
            getFeaturesThumb:function(originUrl){
                return createImage(originUrl,4);
            },
            /**
             * 获取分享的图片
             * @param  {[type]} originUrl 原始图片地址
             * @return {[type]}           图片地址
             */
            getShareThumb:function(originUrl){
                //1.原始图片，6号文件夹
                var url=createImage(originUrl,6);
                return url.replace(".jpg",".png");
            },
            /**
             * 获取电影海报缩略图
             * @param  {[type]} originUrl 原始图片地址
             * @return {[type]}           图片地址
             */
            getVideoPoster:function(originUrl){
                return createImage(originUrl,8)
            },
            /**
             * 三边主题显示的缩略图
             * @param  {[type]} originUrl 原始图片地址
             * @return {[type]}           图片地址
             */
            getSideThumb:function(originUrl){
                return createImage(originUrl,9)
            }

        }
    })
    .directive('pageLoading',function(){
        var template='<ion-spinner icon="ios-small" style="position:absolute;top:50%;left:50%;z-index:2;margin-left:-14px;margin-top:-16px"></ion-spinner>';
        var linker = function(scope, element, attrs) {
            scope.$on('pageLoading.hide',function(){
                element.css({
                    display:'none'
                });
            })
        }
        return {
            restrict: "E",
            link: linker,
            template:template,
            replace: true
        };
    })
