'use strict';
/**
 * @author SuperK
 * @desc 用于打点信息的相关操作
 * @date 2015/05/15
 **/
rootModule
    .factory('DotInfo', function(Side, UUID, ENV, $rootScope) {
        var localDots, shownDots, sideScope; //
        var defaultDot;
        /**
         * 初始化打点相关方法
         * @param  {[type]}
         * @param  {[type]}
         * @return {[type]}
         */
        function __init(scope, watchType, watchId, callback) {
            localDots = [];
            shownDots = [];
            sideScope = scope;
            scope.dots = [];
            if (!scope.$$phase) {
                scope.$apply();
            }
            __getDotInfoList(watchType, watchId, callback);
        }
        /**
         * 加载默认的评论信息条
         */
        function addDefaultCommentDot(watchId) {
            defaultDot = {
                uuid: UUID.newuuid(), //唯一的ID，每个主题
                dotId: watchId, //评论ID
                timePoint: 0, //当前时间点
                hasShow: false,
                visible: false, //是否正在显示,
                inputMode: 0,
                handShow: true,
                type: -1,
                isTop:0,
                /**
                 * 当前的状态 默认'item-nomal'
                 * @type {String} 'item-nomal':常规显示，背景为白色，透明度为1, 默认值
                 *                'item-show' :模块在最前端显示，背景色为白色，带有投影效果，透明度为1
                 *                'item-hide' :模块在最下端显示，背景色为浅灰色，透明度为0.4 无投影效果
                 */
                status: 'item-nomal',
                userStatus:1, //当前主题的状态，1.默认值，2.高亮
                videoId: watchId
            }
            sideScope.dot = defaultDot;
            localDots.push(defaultDot);
        }
        /**
         * 获取
         * @param  {[type]}
         * @param  {[type]}
         * @param  {Function}
         * @return {[type]}
         */
        function __getDotInfoList(watchType, watchId, callback) {
            $rootScope.dotInfoList = {
                videoId: watchId,
                list: localDots
            }
            Side.select_dot_info(watchType, watchId, 0).$promise.then(function(result) {
                addDefaultCommentDot(watchId);
                if (result.success) {
                    //解析数据，对数据进行分类
                    if (result.data && result.data.length > 0) {
                        //对数据进行过滤
                        localDots = [];
                        //默认增加一个评论模块
                        localDots.push(defaultDot);
                        angular.forEach(result.data, function(item, index) {
                            angular.forEach(item.dotInfoList, function(it) {
                                if (it.type == 2 || it.type == 5) {
                                    for (var i = 0; i < it.choice.length; i++) {
                                        //hasnot表示未投票，own表示自己投的票，others表示单选状态时，另一个选项的颜色
                                        it.choice[i].status = 0;
                                        it.choice[i].count = 0;
                                    }
                                }
                                 //初始化部分页面用到的数据

                                var newItem=angular.extend({
                                    uuid: UUID.newuuid(), //唯一的ID，每个主题
                                    dotId: item.dotId, //主题ID
                                    timePoint: parseInt(item.timePoint), //当前时间点
                                    hasShow: false,
                                    visible: false, //是否正在显示,
                                    isTop:item.isTop, //是否置顶
                                    // handShow:true,
                                    status:'item-nomal',
                                    videoId:watchId,
                                    userStatus:item.isTop==1?2:1
                                }, it);
                                if(newItem.type==10){ //简介
                                    newItem.visible=true;
                                }

                                
                                // if(newItem.type == 1){
                                //     newItem.setTop=1;
                                // }else{
                                //     newItem.setTop=0;
                                // }
                                //初始化部分页面用到的数据
                                localDots.push(newItem);
                            })


                        });
                        //根据时间排序
                        localDots = localDots.sort(function(a, b) {
                            return a.timePoint - b.timePoint;
                        });
                        //这里来保存主题列表信息
                        $rootScope.dotInfoList = {
                            videoId: watchId,
                            list: localDots
                        }
                    }
                } else {
                    console.error('加载打点信息失败');
                }
                callback && callback();
            });
        }
        /**
         * 根据时间查询打点信息,这里需要注意如果中途时间被拖动过，不需要重新显示，并且，把之前没显示的打点信息直接显示
         * @param  {[type]} time [description]
         * @return {[type]}      [description]
         */
        function __getShownDotsByTime(time, videoId) {
            var dots = [];
            if (localDots && localDots.length > 0) {
                angular.forEach(localDots, function(item, index) {
                    if (parseInt(item.timePoint) - time < 0 && item.videoId == videoId) {
                        dots.push(item);
                    }
                });
            }
            return dots;
        }
        /**
         * 根据UUID获取打点信息
         * @param  {[type]} dotId [description]
         * @return {[type]}       [description]
         */
        function __getDotByUUID(uuid) {
            var dot;
            angular.forEach(localDots, function(item, index) {
                if (uuid == item.uuid && item.videoId == sideScope.videoInfo.videoId) {
                    dot = item;
                    return false;
                }
            });
            return dot;
        }
        return {
            init: function(scope, watchType, watchId, callback) {
                __init(scope, watchType, watchId, callback);
                return this;
            },
            getShownDotsByTime: function(currentTime,videoId) {
                return __getShownDotsByTime(currentTime,videoId);
            },
            getDotByUUID: function(uuid) {
                return __getDotByUUID(uuid);
            },
            getShownDots: function() {
                return sideScope.dots;
            },
            getLastShownDot: function() {
                var dot = null;
                if (sideScope.dots && sideScope.dots.length > 0) {
                    dot = sideScope.dots[sideScope.dots.length - 1];
                }
                return dot;
            }
        }




    })