'user strict';
/**
 * @description 日志组件
 */
rootModule
    .factory('Log', function (Storage) {
        var log = {
            config: {
                logFileName: "tvHelper.txt",//log file name
                logDirectoryName: "tvHelper",//log directory
                writeContent: ""// write data
            },
            writer: {},//writer
            logFile: {},//file of this log file in memory
            createDirectory: function () {//if not exist,create this directory
                if(typeof LocalFileSystem !='undefined'){
                    window.requestFileSystem(LocalFileSystem.PERSISTENT, 0, function (fileSystem) {
                        log.logFile = fileSystem.root.getDirectory(log.config.logDirectoryName, {
                            create: true,
                            exclusive: false
                        }, function (logFile) {
                            log.logFile = logFile;
                            log.createLogFile();
                        }, function (error) {
                            console.log(error.code);
                        });

                    }, function (error) {
                        console.log("读文件夹失败：错误代码" + error.code);
                    });
                }

            },
            createLogFile: function () {
                //if not exist,create this log file
                if (log.logFile) {
                    log.logFile.getFile(log.config.logFileName, {
                        create: true,
                        exclusive: false
                    }, function (fileEntry) {
                        fileEntry.createWriter(function (writer) {
                            log.writer = writer;
                        }, function (error) {
                            console.log(error.code)
                        });
                    }, function (error) {
                        console.log(error.code)
                    });
                } else {
                    console.log("log.logFile is null");
                }
            }
        };

        return {
            init: function () {
                log.createDirectory();
            },
            //添加日志信息
            addLog: function (group, logInfo) {
                if (log.writer) {
                    var $writer = log.writer;
                    var $log = "log-[" + group + "]" + new Date().getTime() + "\t\t";
                    for (logAttr in logInfo) {
                        $log += "\t\t" + logInfo[logAttr];
                    }
                    //  log("fileName="+writer.fileName+";fileLength="+writer.length+";position="+writer.position);
                    $writer.onwrite = function (evt) {//当写入成功完成后调用的回调函数
                        console.log("write success");
                    };
                    $writer.onerror = function (evt) {//写入失败后调用的回调函数
                        console.log("write error");
                    };
                    $writer.onabort = function (evt) {//写入被中止后调用的回调函数，例如通过调用abort()
                        console.log("write abort");
                    };
                    // 快速将文件指针指向文件的尾部 ,可以append
                    $writer.seek($writer.length);
                    $writer.write($log + "\n");//向文件中写入数据
                    //  writer.truncate(11);//按照指定长度截断文件
                    //  writer.abort();//中止写入文件
                }


            }
        }
    })
    .factory('Toast', function($cordovaToast,ENV){
        return {
            show:function(msg){
                if(typeof window.plugins!='undefined'&&typeof window.plugins.toast!='undefined'){ //使用原生的toast来提示
                    $cordovaToast.showShortBottom(msg);
                }else{
                    //直接在控制台输出
                    console.error('%c'+msg,'font-size:1rem;color:#0091D9')
                }
            },
            showLong:function(msg){
                if(typeof window.plugins!='undefined'&&typeof window.plugins.toast!='undefined'){ //使用原生的toast来提示
                    $cordovaToast.showLongBottom(msg);
                }else{
                    //直接在控制台输出
                    console.error('%c'+msg,'font-size:1rem;color:#0091D9')
                }
            },
            /**
             * Debug的相关信息
             * @param  {[type]} msg [description]
             * @return {[type]}     [description]
             */
            debug:function(msg){
                if(ENV.debug){
                    this.show(msg);
                }else{
                    console.log(msg);
                }
            }
        }
    })