rootModule
    .factory('MediaSvc', function(Toast) {
        var media;
        /**
         * 下载文件
         * @param  {[type]} url [description]
         * @return {[type]}     [description]
         */
        function downloadFile(url, successCallback, errorCallback, progressEventCallback) {
            //这里需要判断是否传入了服务器的地址，如果为服务器地址则判断本地是否存储了缓存文件，
            //1.如果没有则需要下载
            //2.如果有则直接将地址修改为本地地址
            //1.获取到文件名称
            //1.判断是否为一个远程地址
            if (url.indexOf("http") > -1) {
                if (typeof LocalFileSystem != 'undefined') {
                    var filename = url.substring(url.lastIndexOf("/") + 1, url.length)
                        //2.判断文件是否存在，如果不存在需要下载，如果存在，直接播放
                    window.requestFileSystem(LocalFileSystem.PERSISTENT, 0, function(fileSystem) {
                        fileSystem.root.getFile("/handsight/tvhelper/cache/" + filename, null, function(fileEntry) {
                            fileEntry.file(function(file) {
                                //文件存在
                                console.log('文件存在', file.size);
                                var path=fileEntry.toURL();
                                if(device.platform=='iOS'){
                                    path=path.replace("file://","");
                                }
                                progressEventCallback && progressEventCallback(1,path);
                                successCallback(path);
                            }, function() {
                                //文件不存在
                                console.log('文件不存在，需要下载');
                                download(url, filename, successCallback, errorCallback, progressEventCallback);
                            });
                        }, function(error) {
                            console.log('文件不存在，需要下载getFileEntry Error', error);
                            download(url, filename, successCallback, errorCallback, progressEventCallback);
                        });
                    }, function(error) {
                        //文件错误
                        console.log('getFileSystem Error');
                    });
                } else {
                    progressEventCallback && progressEventCallback(1,url);
                    successCallback(url);
                }
            } else {
                successCallback(url)
            }
        }
        /**
         * 获取当前缓存的目录
         * @param  {Function} callback [description]
         * @return {[type]}            [description]
         */
        function getCacheFullPath(callback) {
            if (window.fullPath && window.fullPath != "") {
                callback && callback(window.fullPath);
            } else {
                window.fullPath = "";
                //这里来初始化创建缓存文件夹
                try {
                    //查找是否有download这个文件夹，没有则创建，然后找到这个文件夹的绝对路径
                    window.requestFileSystem(LocalFileSystem.PERSISTENT, 0, function(fileSystem) {
                        fileSystem.root.getDirectory("handsight", {
                            create: true
                        }, function() {});
                        fileSystem.root.getDirectory("/handsight/tvhelper", {
                            create: true
                        }, function() {});
                        fileSystem.root.getDirectory("/handsight/tvhelper/cache/", {
                            create: true
                        }, function(entry) {
                            //网上流传的资料中都是使用fullPath，在这里我获取到的是相对目录，在下载时使用会报错，所以换做了toURL()
                            //这是一个全局全局变量，用以保存路径
                            fullPath = entry.toURL();
                            if(device.platform=='iOS'){
                                fullPath=fullPath.replace("file://","");
                            }
                            callback && callback(fullPath);
                        }, function() {
                            Toast.show('访问存储空间失败code:1');
                        });
                    }, function(e) {
                        Toast.show('访问存储空间失败code:2');
                    });
                } catch (e) {
                    Toast.show('访问存储空间失败code:3');
                }
            }
        }

        /**
         * 下载文件
         * @param  {[type]} url                   [description]
         * @param  {[type]} filename              [description]
         * @param  {[type]} successCallback       [description]
         * @param  {[type]} errorCallback         [description]
         * @param  {[type]} progressEventCallback [description]
         * @return {[type]}                       [description]
         */
        function download(url, filename, successCallback, errorCallback, progressEventCallback) {
            getCacheFullPath(function(cachePath) {
                var fileTransfer = new FileTransfer();
                var uri = encodeURI(url);
                var filePath = cachePath + filename;
                fileTransfer.download(
                    uri,
                    filePath,
                    function(fileEntry) {
                        successCallback(filePath);
                        console.log("download complete: " + filePath);
                    },
                    function(error) {
                        errorCallback(error);
                        console.log("download error source " + url);
                        console.log("download error target " + filePath);
                        console.log("download error code" + error.code);
                    },
                    false, {}
                )
                fileTransfer.onprogress = function(progressEvent) {
                    if (progressEvent.lengthComputable) {
                        progressEventCallback && progressEventCallback(progressEvent.loaded / progressEvent.total,filePath);
                    } else {
                        console.log('加载完成')
                    }
                }
            })
        }

        return {
            //播放音频
            play: function(url, successCallback, errorCallback, playCallback, playMediaStatus) {
                downloadFile(url, function(filePath) {
                    if (typeof Media != 'undefined') {
                        media = new Media(filePath, function() {
                            console.log('播放成功');
                            playMediaStatus && playMediaStatus(4);
                        }, function(err) {
                            console.log(err, "");
                            errorCallback && errorCallback(err);
                        }, playMediaStatus)
                        media.play();
                        playCallback && playCallback(media);
                    } else {
                        var audio = document.createElement("AUDIO");
                        audio.src = filePath;
                        audio.load();
                        audio.play();
                        playCallback&&playCallback(audio);
                        audio.addEventListener('ended',function(){
                            playMediaStatus&playMediaStatus(4);
                        })                       
                        audio.addEventListener('error',function(){
                            playMediaStatus&playMediaStatus(4);
                        })                        
                        //插件不存在
                        console.log('media plugin not found');
                        //successCallback && successCallback('media plugin not found');
                    }
                }, function(error) {
                    console.log(error);
                });
            },
            /**
             * 缓冲数据
             * @param  {[type]}   url      url地址
             * @param  {Function} callback 回调函数
             * @return {[type]}            [description]
             */
            bufferFile: function(url, callback, progressEventCallback) {
                downloadFile(url, function(filepath) {
                    console.log('file load complete');
                    console.log('serverURL:' + url);
                    console.log('localURL:' + filepath);
                    callback && callback(filepath)
                }, function(err) {
                    console.log('serverURL:' + url);
                    console.log('localURL:' + filepath);
                    console.log('下载文件出现错误：' + JSON.stringify(err));
                }, progressEventCallback)
            },
            /**
             * [createAudioPlayer description]
             * @param  {[type]} filePath                文件路径
             * @param  {[type]} playSuccessCallback     播放成功回调函数
             * @param  {[type]} playErrorCallback       播放错误回调函数
             * @param  {[type]} playMediaStatusCallback 操作状态发生改变回调函数
             * @return {[type]}                         [description]
             */
            createAudioPlayer: function(filePath, playSuccessCallback, playErrorCallback, playMediaStatusCallback) {
                var player = null;
                //1.判断Cordova Media插件是否存在
                if (typeof Media != 'undefined') {
                    player = new Media(filePath, playSuccessCallback, playErrorCallback, playMediaStatusCallback);
                } else {
                    //插件不存在
                    console.log('Cordova media plugin not installed');
                    var audio = document.createElement("AUDIO");
                    audio.src = filePath;
                    audio.load();
                    player = audio;
                    player.seekTo = function(currentTime) {
                        audio.currentTime = currentTime;
                    }
                    player.getDuration = function() {
                        return audio.duration;
                    }
                    player.release = function() {
                        audio = null;
                        player = null;
                    }
                }
                return player;
            }
        }
    })