/**
 * @author  Superk
 * @date 2016/01/20
 * @description  用于通知和推送相关的服务
 */
rootModule.factory('NotifySvc', function(ENV, $resource, $ionicPlatform, Movies, $state) {
    var deviceToken;
    var resource = $resource(ENV.api + '/notify', null, {
        //绑定终端接口
        addNotify: {
            method: 'POST',
            url: ENV.api + '/message/add'
        },
        saveDeviceToken: {
            method: "POST",
            url: ENV.api + '/devicetoken/add'
        }
    })

    /**
     * 添加通知推送提醒，目前仅支持场次提醒
     * @param {[type]} session 场次信息
     */
    function addNotify(session) {
        var notifyData = {
                //标题
                title: session.movieName,
                //副标题
                secondTitle: session.room + "  " + session.starttime + "开播了，点击进入精彩互动",
                type: 1, //1.	场景即将开始的提示提示,2.	电影即将上映的提示,3.	观影后的提示,4.	更新版本的提示
                isShowRed: 1, //是否显示小红点
                //时间
                sendTime: session.showtime,
                content: session.senceId,
                deviceToken: deviceToken
            }
            //1.Android版本需要将消息添加到后台系统中
        if (typeof HSPlugin != "undefined") {
            if ($ionicPlatform.is("Android")) {
                notifyData.id=notifyData.content;
                HSPlugin.execFunc('addNotify', [JSON.stringify(notifyData)], function() {
                    console.log('发送给Android后台服务成功')
                }, function() {
                    console.log('发送给Android后台服务失败')
                });
            } else {
                //IOS无需
            }
        }
        //2.将消息保存到服务端
        resource.addNotify(null, param(notifyData)).$promise.then(function(data) {
            console.log('推送消息保存到服务器成功！')
        }, function(err) {
            console.log('推送消息保存到服务器失败！')
        })
        console.log(session);
    }
    /**
     * 点击状态栏回调的接口函数
     * @param  {[type]} data 数据
     * @return {[type]}      [description]
     */
    function notifyCallback(data) {
        //解析数据
        if (typeof data == 'string') {
            data = JSON.parse(data);
        } else {
            //格式正确无需转换
        }
        //开始组织数据
        switch (data.type) {
            case 1: //电影场景的提示
                goToSide(data);
                break;
            default:
                //暂时不处理
                console.log('暂时不支持该类型' + data.type);
                break;
        }
    }
    /**
     * 跳转进入三边页面
     * @return {[type]} [description]
     */
    function goToSide(data) {
        //解析相关的数据
        var sessionId = data.content; //场次ID
        Movies.getSenceInfor(sessionId, function(data) {
            var currentD=new Date();

            var startT=new Date(data.realtime.replace(/-/g,"/"));
            var endT=new Date(data.movieEndTime.replace(/-/g,"/"));
            //查询出场次的相关信息
            var videoId=data.videoId; //视频ID

            //视频已经播放的时间
            var startedTime = parseInt((currentD - startT)/1000);
            //视频的片长
        	var movieLength = parseInt((endT - startT)/1000);
        	//视频名称
        	var movieName=data.movieName;
        	//视频开始时间
        	var startTime=startT;
            //跳转进入三边
            $state.go('platform-side', {
                videoId: videoId, //跳转传入一个视频ID
                tvhelper: "tvhelper",
                startedTime: startedTime,
                movieLength: movieLength,
                moviename: encodeURIComponent(moviename),
                startTime: startTime,
                isSystemCountdown: true
            });
        })
    }
    /**
     * 初始化推送相关的内容
     * @return {[type]} [description]
     */
    function initNotify() {
        if (typeof HSPlugin != "undefined") {
            HSPlugin.notifyCallback = notifyCallback;
            //2.保存DeviceToken
            if ($ionicPlatform.is("iOS")) {
                HSPlugin.execFunc('getDeviceToken', [], function(data) {
                    deviceToken = data;
                    resource.saveDeviceToken(null, param({
                        deviceToken: deviceToken
                    })).$promise.then(function() {
                        //保存DeviceToken成功
                    }, function(er) {
                        console.log('保存DeviceToken失败')
                    })
                }, function(err) {
                    //获取DeviceToken失败
                })
            }
        } else {
            console.log("HSPlugin is not defined")
        }
    }
    return {
        addNotify: addNotify, //用于添加一条通知提醒
        notifyCallback: notifyCallback,
        initNotify: initNotify //初始化
    }
})