'use strict';
/**
 * @description 拍照上传
 * @author Vivian
 * @date 2015-05-15
 */
rootModule
    .factory('photoService', function (upload, Toast, $ionicLoading) {
        return {
            /**
             * 用户拍照上传图片
             * @param  {[type]} successCallback 成功回调函数
             * @param  {[type]} errorCallback   失败回调函数
             * @return {[type]}                 [description]
             */
            takePhoto: function (successCallback, errorCallback) {
                if (typeof navigator.camera != 'undefined') {
                    navigator.camera.getPicture(onSuccess, onFail, {
                        quality: 20, //设置画质，质量
                        destinationType: Camera.DestinationType.FILE_URI, //设置返回的数据，Base64 、文件路径
                        encodingType: Camera.EncodingType.JPEG, //文件编码格式
                        allowEdit: true //是否允许编辑
                        // targetWidth:1280, //图片输出宽度
                        // targetHeight:1280 //图片输出高度
                    });
                } else {
                    Toast.show('拍照模块不存在');
                }
                /**
                 * 拍照成功调用的回调函数
                 * @param  {[type]} imageURI 返回的文件路径
                 * @return {[type]}          [description]
                 */
                function onSuccess(imageURI) {
                    //调用文件上传服务
                    upload.uploadEvent(imageURI, {
                        type: '3',
                        fileType: 'img'
                    }, function (result) {
                        successCallback(result);
                    }, function (err) {
                        errorCallback(err);
                    });
                }

                /**
                 * 拍照失败的回调函数
                 * @param  {[type]} message 错误消息
                 * @return {[type]}         [description]
                 */
                function onFail(message) {
                    errorCallback(message);
                }
            },
            /**
             * 图片选择服务
             * @param  {[type]} successCallback 成功回调函数
             * @param  {[type]} errorCallback   失败回调函数
             * @return {[type]}          [description]
             */
            choiceImg: function (successCallback, errorCallback) {
                var options = {
                    maximumImagesCount: 1,
                    width: 800,
                    height: 800,
                    quality: 80
                };
                //调用图片选择服务
                if (typeof imagePicker != 'undefined') {
                    //开始选择图片
                    imagePicker.getPictures(function (results) {
                        //选择图片上传
                        upload.uploadEvent(results[0], {
                            type: '3',
                            fileType: 'img'
                        }, function (result) {
                            successCallback(result);
                        })
                    }, function (error) {
                        errorCallback(err);
                    }, options);
                } else {
                    Toast.show('图片选择组件不存在');
                }
            },
            takePhotoForHead: function (callback, errorCallback) {
                navigator.camera.getPicture(onSuccess, onFail, {
                    quality: 20,
                    destinationType: Camera.DestinationType.FILE_URI,
                    encodingType: Camera.EncodingType.JPEG,
                    allowEdit: true,
                    cameraDirection: navigator.camera.Direction.FRONT
                    // targetWidth:1280, //图片输出宽度
                    // targetHeight:1280 //图片输出高度
                });

                function onSuccess(imageURI) {
                    //console.log("takePhotoForHead:onSuccess" + imageURI);
                    //var image = document.getElementById('image');
                    //image.src = "data:image/jpeg;base64," + imageData;
                    callback(imageURI);
                }

                function onFail(message) {
                    errorCallback(message);
                    // alert('Failed because: ' + message);
                    //console.log("takePhotoForHead:onFail" + message);
                }
            }
        }
    });