/**
 * 娱乐相关的数据服务
 * @param  {[type]} rootModule){	function Play(ENV){	}	rootModule.service('Play',Play);} [description]
 * @return {[type]}                        [description]
 */
rootModule
    .factory('PlaySvc', function(ENV, $resource) {
        var filterTypes = [{
            id: 0,
            text: '全部',
            active: true
        }, {
            id: 1,
            text: '配音',
            active: false
        }, {
            id: 2,
            text: '配词',
            active: false
        }];
        var resource = $resource(ENV.api + "/interface/rhapp/recreation/getlist", null, {
            //1.1   查询娱乐成果列表
            getList: {
                method: "GET",
                url: ENV.api + '/interface/rhapp/recreation/getlist'
            },
            getSigleDisportInfo: {//查询当个成果
                method: "GET",
                url: ENV.api + '/interface/rhapp/recreation/get'
            },
            addRecreation:{ //添加成果接口
                method:'POST',
                url:ENV.api+'/interface/rhapp/recreation/add'
            },
            updateRecreation:{ //修改成果
                method:'POST',
                url:ENV.api+'/recreation/update'
            },
            deleteRecreation:{ //删除成果
                method:'POST',
                url:ENV.api+'/interface/rhapp/recreation/delete'
            },
            releaseRecreation:{ //发布成果
                method:'POST',
                url:ENV.api+'/interface/rhapp/recreation/release'
            },
            getDisportData:{ //查询单个素材
                method:'POST',
                url:ENV.api+'/interface/rhapp/recreation/material/get'
            },
            getDisportDataList:{ //查询娱乐素材列表
                method:'POST',
                url:ENV.api+'/interface/rhapp/recreation/material/getlist'
            }
        });
        return {
            getList: function(id, recreationType, disportDataId, type, page, isApproved, callback) {
                resource.getList({
                    id: id,
                    recreationType: recreationType,
                    disportDataId: disportDataId,
                    type: type,
                    page: page,
                    isApproved: isApproved
                }, null, function(respones) {
                    callback && callback(respones);
                })
            },
            getFilterTypes: function() {
                return filterTypes;
            },
            /**
             * 查询单个娱乐成果信息
             * @param  {[type]} disportId 娱乐成果ID
             * @return {[type]}           [description]
             */
            getSigleDisportInfo: function(disportId,callback) {
                resource.getSigleDisportInfo({
                    id: disportId
                }, null, function(respones) {
                    callback && callback(respones);
                })
            },
            /**
             * 添加成果
             * @param {[type]}   disportDataId 娱乐素材ID
             * @param {[type]}   disportTitle  娱乐成果标题
             * @param {[type]}   content       内容（json字符串）
             * @param {Function} callback      [description]
             */
            addRecreation:function(disportDataId,disportTitle,content,callback){
                return resource.addRecreation(null, param({
                    disportDataId: disportDataId,
                    disportTitle: disportTitle,
                    content: content
                })).$promise.then(function(result){
                    callback(result);
                })
            },
            /**
             * 更新成果
             * @param  {[type]}   disportDataId 娱乐素材ID
             * @param  {[type]}   disportTitle  娱乐成果标题
             * @param  {[type]}   content       娱乐成果内容
             * @param  {Function} callback      [description]
             * @return {[type]}                 [description]
             */
            updateRecreation:function(disportId,disportTitle,content,callback){
                var json=JSON.stringify({
                    disportId:disportId,
                    disportTitle:disportTitle,
                    content:content
                })
                return resource.updateRecreation(null, param({
                    json:json
                })).$promise.then(function(result){
                    callback(result);
                })
            },
            /**
             * 删除成果
             * @param  {[type]}   disportId 成果ID
             * @param  {Function} callback  [description]
             * @return {[type]}             [description]
             */
            deleteRecreation:function(disportId,callback){
                return resource.deleteRecreation(null, param({
                    disportId:disportId
                })).$promise.then(function(result){
                    callback(result);
                })
            },
            /**
             * 发布成果
             * @param  {[type]}   disportId 成果id
             * @param  {Boolean}  isRelease 发布与否（0：取消发布，1：发布）未传值时默认为标记发布
             * @param  {Function} callback  [description]
             * @return {[type]}             [description]
             */
            releaseRecreation:function(disportId,isRelease,callback){
                return resource.releaseRecreation(null, param({
                    disportId:disportId,
                    isRelease:isRelease
                })).$promise.then(function(result){
                    callback(result);
                })
            },
            /**
             * 查询娱乐素材
             * @param  {[type]}   disportDataId [description]
             * @param  {Function} callback      [description]
             * @return {[type]}                 [description]
             */
            getDisportData:function(disportDataId,callback){

                return resource.getDisportData(null, param({
                    disportDataId:disportDataId
                })).$promise.then(function(result){
                    callback(result);
                })
            },
            /**
             * 查询娱乐素材列表
             * @param  {[type]}   videoId 视频ID
             * @param  {[type]}   type 素材类型（0：所有；1：配音素材；2：配台词素材；3：配图素材）
             * @param  {Function} callback      [description]
             * @return {[type]}                 [description]
             */
            getDisportDataList:function(videoId,type,callback){
                return resource.getDisportDataList(null, param({
                    vid:videoId,
                    type:type
                })).$promise.then(function(result){
                    callback(result);
                })
            },
        }
    })