'user strict';
/**
 * @description 三边｛边看、边搜、边买｝
 * @author SuperK
 * @date 2015/04/07
 */
rootModule
    .factory('Side', function(ENV, $resource, $log, $http, $q, Storage) {
        var scdx_interface = ""
        var comments = []; //存储视频评论相关信息
        var commentPoint = []; //存储评论相关信息
        var resource = $resource(ENV.api, null, {
            //通过视频ID查询打点相关信息（包含：话题、商品、投票）接口
            select_dot_info: {
                method: 'GET',
                url: ENV.api + scdx_interface + '/side/select_dot_info'
            },
            //获取主题列表接口（包含：话题、商品、投票）
            get_theme: {
                method: 'POST',
                url: ENV.api + scdx_interface + '/side/get_theme'
            },
            //发表主题的聊天信息接口
            add_theme_talk: {
                method: 'POST',
                url: ENV.api + scdx_interface + '/poll/side/add_theme_talk'
            },
            //节目快照接口
            get_thumb: {
                method: 'POST',
                url: ENV.api + scdx_interface + '/side/get_thumb'
            },
            //初始化拉取相关评论信息
            get_theme_talk: {
                method: 'GET',
                url: ENV.api + scdx_interface + '/side/select_theme_talk'
            },
            //查询消息监听端口
            get_listener_port: {
                method: 'GET',
                url: ENV.api + scdx_interface + '/side/get_listener_port',
                timeout: '2000'
            },
            //查询历史的消息列表
            getAllHistoryComments: {
                method: 'POST',
                url: ENV.api + scdx_interface + '/side/select_theme_history_talk'
            },
            //根据评论ID、点赞ID、投票ID查询信息
            getCommentById: {
                method: 'POST',
                url: ENV.api + scdx_interface + '/side/select_comment'
            },
            getDeviceStatus: {
                method: "POST",
                url: ENV.api + scdx_interface + '/side/get_device_status',
                timeout:20000,
                interceptor: { //显示超时。
                    response: function(data) {
                        if (data.resource.success) {
                            return data.resource;
                        } else {
                            return data;
                        }
                    }
                }
            },
            sideSearch: {
                method: 'POST',
                url: ENV.api + scdx_interface + '/recipe/get_result_info',
                timeout: 100000,
                interceptor: { //显示超时。
                    response: function(data) {
                        if (data.resource.success) {
                            return data.resource;
                        } else {
                            return data;
                        }
                    }
                }
            },
            longPoll: {
                method: 'POST',
                url: ENV.longPoll + scdx_interface + '/poll/get_comment_poll',
                timeout:2<<20,
                interceptor: { //显示超时。
                    response: function(data) {
                        if (data.resource.success) {
                            return data.resource;
                        } else {
                            return data;
                        }
                    }
                }
            }

        });

        return {
            /**
             * 通过视频ID查询打点相关信息（包含：话题、商品、投票）
             * @param  {[int]} type  观看类型（直播为1，点播为2）
             * @param {[int]} mvid 频道ID或视频ID
             * @param {[int]} themeType 主题类型(1:话题；2：投票，3：商品，0:所有的)
             * @return {[JSON]}          视频打点信息列表
             */
            select_dot_info: function(type, mvid, themeType) {
                return resource.select_dot_info({
                    type: type,
                    mvId: mvid,
                    themeType: themeType
                })
            },
            /**
    		 * 获取主题列表（包含：话题、商品、投票）
    		 * @param  {[int]} videoID  视频ID
             
    		 * @return {[JSON]} 主题列表
    		 */
            get_theme: function(videoId) {
                return resource.get_theme({
                    video_id: video_id
                })
            },
            /**
             * 添加评论、点赞、回复、投票等消息接口
             * @param {[type]} type      对谁发表的内容，即谁的评论（0：视频；1：评论点赞；2：投票；3：话题；4：评论(回复)；5：频道 ）
             * @param {[type]} commentId 被评论人ID(视频ID或话题ID或评论ID或投票ID或频道ID)
             * @param {[type]} rootId    评论时的根主题Id：(视频ID或话题ID或投票ID或频道ID)
             * @param {[type]} content   内容,随type值的变化，存的内容不同（type值为0、3、4、5：文字，图片，语音；type值为1：对评论的点赞值；type值为2：选项ID；）
             * @param {[type]} deviceId  设备ID
             */
            add_theme_talk: function(type, commentId, rootId, themeType, content, deviceId, playTime) {
                return resource.add_theme_talk(null, param({
                    type: type,
                    comentId: commentId,
                    rootId: rootId,
                    content: content,
                    deviceId: deviceId,
                    themeType: themeType,
                    playTime: playTime
                }))
            },
            /**
             * 节目快照接口
             * @param  {[type]} type  观看类型（直播为1，点播为2）
             * @param  {[type]} mvid  频道ID或视频ID
             * @param  {[type]} count 几张图（当前进度的前后几张图）
             * @return {[type]}       {time,thumbImg,bigImg}
             */
            get_thumb: function(type, mvid, count, deviceId) {
                return resource.get_thumb({
                    mvid: mvid,
                    type: type,
                    count: count,
                    deviceId: deviceId
                }, null)
            },
            /**
             * 根据时间进度获取打点信息
             * @param  {[type]} time [description]
             * @return {[type]}      [description]
             */
            getDotByTime: function(time) {

            },
            getThemeTalk: function(type, count, rootId, deviceId, themeType) {
                return resource.get_theme_talk({
                    type: type,
                    count: count,
                    rootId: rootId,
                    deviceId: deviceId,
                    themeType: themeType
                })
            },
            /**
             * 获取视频评论信息
             * @param  {[type]}   type     [description]
             * @param  {[type]}   count    [description]
             * @param  {[type]}   rootId   [description]
             * @param  {[type]}   deviceId [description]
             * @param  {[type]}   themeType[description]
             * @param  {Function} callback [description]
             * @return {[type]}            [description]
             */
            getComments: function(type, count, rootId, deviceId, themeType, callback) {
                this.getThemeTalk(type, count, rootId, deviceId, themeType).$promise.then(function(result) {
                    if (result.success) {
                        comments = result.data;
                        callback && callback(comments);
                    } else {
                        callback && callback([]);
                    }
                })
            },
            /**
             * 通过cid 获取评论信息
             * @param  {[type]} cid [description]
             * @return {[type]}     [description]
             */
            getCommentByCid: function(cid) {
                var comment = null;
                if (comments && comments.length > 0) {
                    angular.forEach(comments, function(item) {
                        if (item.cid = cid) {
                            comment = item;
                            return false;
                        }
                    })
                } else {
                    comment = null;
                }
                return comment;
            },
            /**
             * 重置数据
             * @return {[type]} [description]
             */
            resetData: function() {
                comments = [];
            },
            /**
             * 获取comet4J的侦听端口名称
             * @param  {[type]} mvid 频道ID或视频ID
             * @param  {[type]} type 类型(直播为1，点播为2)
             * @return {[type]}      [description]
             */
            getListenerPort: function(mvid, type) {
                return resource.get_listener_port({
                    mvId: mvid,
                    type: type
                })
            },
            /**
             * 通过频道或视频ID查询历史评论信息
             * @param  {[type]} rootId 评论时的根主题Id：(视频ID或话题ID或投票ID或频道ID)
             * @param  {[type]} themeType 哪种主题,主题类型（1:话1题2：投票3：商机-1：视频 -2：频道 ）
             * @param  {[type]} callback [description]
             * @return {[type]}      [description]
             */
            getAllHistoryComments: function(rootId, themeType, callback) {
                commentPoint = [];
                resource.getAllHistoryComments(null, param({
                    themeType: themeType,
                    rootId: rootId
                })).$promise.then(function(data) {
                    if (data.success) {
                        commentPoint = data.data;
                    }
                    callback && callback(data);
                })
            },
            getCommentById: function(cId) {
                return resource.getCommentById(null, param({
                    cId: cId
                }))
            },
            /**
             * 获取终端的当前状态
             * @param  {[type]} deviceId 终端设备ID
             * @return {[type]}          [description]
             */
            getDeviceStatus: function(deviceId) {
                return resource.getDeviceStatus(null, param({
                    deviceId: deviceId
                }));
            },
            /**
             * 三边搜索功能
             * @param  {[type]} keyword [description]
             * @return {[type]}         [description]
             */
            sideSearch: function(keyword, index, uuid) {
                return resource.sideSearch(null, param({
                    keyword: keyword,
                    recipeId: 72,
                    level: 0,
                    index: index,
                    uuid: uuid
                }))
            },
            longPoll: function(mvid, type) {
                return resource.longPoll(null, param({
                    mvId: mvid,
                    type: type
                }))
            }
        }
    })