'use strict';
/**
 * @description 电视终端相关功能Services
 * @author SuperK
 * @date 2015-04-02
 * @name hand.services:TVService
 */
rootModule
    .factory('TV', function(ENV, $resource, $log, $q, Storage) {
        var resource = $resource(ENV.api + '/device', null, {
            //绑定终端接口
            add_device: {
                method: 'POST',
                url: ENV.api + '/device/add_device'
            },
            //移除终端接口
            delete_device: {
                method: 'GET',
                url: ENV.api + '/device/remove_device'
            },
            //设置终端别名接口
            set_device_name: {
                method: 'POST',
                url: ENV.api + '/device/set_device_name'
            },
            //获取终端主人信息
            get_device_userInfo: {
                method: 'POST',
                url: ENV.api + '/device/get_user_info'
            },
            //获取终端列表接口
            get_devices: {
                method: 'GET',
                url: ENV.api + '/device/get_devices',
                timeout: 100000,
                interceptor: {
                    response: function(data) {
                        console.log('response interceptor', data);
                        if (data.resource.success) {
                            return data.resource;
                        } else {
                            return data;
                        }
                    }
                }
            },
            //获取终端详情接口
            get_device_info: {
                method: 'GET',
                url: ENV.api + '/device/get_device_info'
            },
            //请求二维码图片接口
            get_qrcode_img: {
                method: 'POST',
                url: ENV.api + '/device/get_qrcode_img'
                // url:ENV.api + '/interface/zzqj/get_device_img'
            },
            //扫描二维码链接好友设备接口
            add_friend_device: {
                method: 'POST',
                url: ENV.api + '/device/add_friend_device'
            },
            //开放和关闭好友是否连接接口
            on_off_friend: {
                method: 'POST',
                url: ENV.api + '/device/on_off_friend'
            }
        })
        return {
            /**
             * 获取我的终端列表
             * @return {[type]} 终端列表
             */
            get_devices: function() {
                return resource.get_devices();
            },
            /**
             * 绑定一个终端
             * @param {[type]} device  终端信息
             */
            add_device: function(device) {
                return resource.add_device(null, param(device));
            },
            /**
             * 解绑一个终端
             * @param {[type]} deviceID  终端信息
             */
            delete_device: function(deviceId, userId) {
                return resource.delete_device({
                    deviceId: deviceId,
                    userId: userId
                });
            },
            /**
             * 根据终端Id获取主人信息
             * @param  deviceId 终端ID
             */
            get_device_userInfo: function(deviceId) {
                return resource.get_device_userInfo(null, param(deviceId));
            },
            /**
             * 设置终端别名
             * @param {[type]} device 设置状态[成功 or 失败]
             */
            set_device_name: function(device) {
                return resource.set_device_name(null, param(device));
            },
            /**
             * 根据终端ID获取终端详细信息
             * @param  {[type]} deviceId 终端ID
             * @return {[type]}           设备详细信息
             */
            get_device_info: function(deviceId) {
                return resource.get_device_info({
                    deviceId: deviceId
                });
            },
            /**
             * 获取添加好友设备二维码图片
             * @param  {[type]} deviceId 终端ID
             * @return {[type]}           二维码图片地址相关信息
             */
            get_qrcode_img: function(deviceId) {
                return resource.get_qrcode_img(null, param({
                    deviceId: deviceId
                }));
            },
            /**
             * 添加好友设备
             * @param {[type]} qrcode   二维码
             */
            add_friend_device: function(qrcode) {
                return resource.add_friend_device(null, param({
                    qrcode: qrcode
                }))
            },
            /**
             * 开关好友连接终端状态
             * @param  {[type]} userId    用户ID
             * @param  {[type]} type      操作类型［断开or 连接］
             * @param  {[type]} deviceId 终端ID
             * @param  {[type]} time      设置连接时长
             * @return {[type]}           [description]
             */
            on_off_friend: function(userId, type, deviceId, time) {
                return resource.on_off_friend(null, param({
                    userId: userId,
                    type: type,
                    deviceId: deviceId,
                    time: time
                }))
            }
        }
    })
    .factory('barCodeScannerUtil',
        function(TV, $ionicLoading, $state, $rootScope, $location, ENV, $cordovaBarcodeScanner, Movies, Toast) {

            /**
             * 解析二维码扫描的内容
             *
             */
            function extractCode(codeStr) {
                if (codeStr && codeStr.indexOf("service.handsight") > -1&&codeStr.indexOf("code=")>-1) {
                    var reg = new RegExp("(^|&)code=([^&]*)(&|$)", "i");
                    var imageDataText = codeStr;
                    var codeText = imageDataText.substr(imageDataText.indexOf('?') + 1).match(reg);
                    var code = "";
                    if (codeText != null) {
                        code = unescape(codeText[2]);
                    }
                    console.log(code);
                    TV.add_friend_device(code).$promise.then(function(data) {
                        try {
                            if (data.success) {
                                $ionicLoading.show({
                                    template: data.msg,
                                    noBackdrop: true,
                                    duration: 3000
                                });
                                $rootScope.$broadcast('reload_devices');
                                $state.go("tab.tv");
                            } else {
                                $ionicLoading.show({
                                    template: data.msg,
                                    noBackdrop: true,
                                    duration: 3000
                                });
                            }
                        } catch (e) {
                            alert(e);
                        }
                    });
                } else if (codeStr && codeStr.indexOf("http") > -1) {
                    window.open(codeStr, '_blank', 'location=yes', 'closebuttoncaption=Done');
                }
            }

            return {
                openSysCamera: function() {
                    //定义一些页面显示文字
                    var param = {
                        text_title: "扫描二维码",
                        text_instructions: '将二维码放入框内即可自动扫描'
                    }
                    $cordovaBarcodeScanner.scan().then(function(barcodeData) {
                        // Success!s Barcode data is here
                        extractCode(barcodeData.text);
                    }, function(error) {
                        // An error occurred
                    });
                },
                /**
                 * 解析二维码、视频、海报，的数据
                 * @param  {[type]} jsonData [description]
                 * @return {[type]}          [description]
                 */
                doScannerResult: function(jsonData) {
                    var obj = JSON.parse(jsonData);
                    switch (parseInt(obj.typeId)) {
                        case 1:
                            extractCode(obj.qrCode);
                            break;
                        case 2:
                            //这里需要区分是视频还是海报
                            if(obj.videoType=='video'){
                              //跳转到三边页面
                              //进入三边
                            var video = obj;
                            var time = video.videoTime.split(':');
                            var startedTime = parseInt(time[0]) * 60 * 60 + parseInt(time[1]) * 60 + parseInt(time[2]);
                            $ionicLoading.show();
                            Movies.getVideoDetails(video.videoCode, function(data) {
                                if (data.success&&data.data) {
                                    var movieLength = data.data.releaseTime;
                                    $state.go('platform-side', {
                                        videoId: data.data.videoId, //跳转传入一个视频ID
                                        tvhelper: "tvhelper",
                                        startedTime: startedTime,
                                        movieLength: movieLength,
                                        moviename: encodeURIComponent(data.data.videoName),
                                    });
                                } else {
                                    Toast.show('影片不存在！'+video.videoName)
                                }
                                $ionicLoading.hide();
                            });
                            }else if(obj.videoType=='picture'){
                              //跳转到视频详情页
                              var videoId = obj.videoCode;
                              $state.go('movie-detail',{ //跳转到视频详情页
                                id:videoId
                              })
                            }else{
                              Toast.show('没有该类型')
                            }
                            break;
                        default:
                            break;
                    }
                }
            }
        }
);