'use strict';
/**
 * @description 用户信息相关服务
 * @author SuperK
 * @date 2015-03-31
 * @{@link  http://www.woaiso.com}
 * @name  hand.services:UserService
 */
rootModule
    .factory('User', function(ENV, $resource, $log, $q, Storage, $ionicPopup, Toast, $ionicLoading, $rootScope) {

        var userInfoKey = 'user_info';
        var userInfoLogin = 'userinfo_login_key';
        var urlInterface = "/interface/rhapp";
        var loginTypeKey="userLoginTypeKey"
        var resource = $resource(ENV.api + '/login', null, {
            get_verifycode: {
                method: 'get',
                url: ENV.api + '/access/get_verifycode'
            },
            signup: {
                method: 'post',
                url: ENV.api + '/user/register'
            },
            login: {
                method: 'POST',
                url: ENV.api + '/user/login',
                timeout: '10000'
            },
            logout: {
                method: 'post',
                url: ENV.api + '/user/logout',
                timeout: 5000,
                interceptor: { //显示超时。
                    response: function(data) {
                        if (data && data.resource.success) {
                            return data.resource;
                        } else {
                            return data;
                        }
                    }
                }
            },
            update_user_info: {
                method: 'POST',
                url: ENV.api + '/user/update'
            },
            get_user_info: {
                method: 'POST',
                url: ENV.api + '/user/get_user_info'
            },
            update_password: {
                method: 'POST',
                url: ENV.api + '/user/update_password'
            },
            commit_suggestion: {
                method: 'POST',
                url: ENV.api + '/user/feedback'
            },
            getAppId: {
                method: 'POST',
                url: ENV.api + '/user/appid'
            },
            thirdLogin: {
                method: "POST",
                url: ENV.api + '/user/third/login'
            }
        });
        /**
         * 对密码进行加密
         * @return {[type]} [description]
         */
        function encodePassword(password) {
            return strEnc(password.toString(), ENV.descKey, ENV.descKey, ENV.descKey);
        }
        return {
            /**
             * 给密码加密
             * @type {[type]}
             */
            encodePassword: encodePassword,
            /**
             * 获取手机校验码
             * @param  {[type]} phone [description]
             * @return {[type]}       [description]
             */
            get_verifycode: function(phone) {
                return resource.get_verifycode({
                    phone: phone
                });
            },
            /**
             * 用户登录
             * @param  {[type]}   username          用户名
             * @param  {[type]}   password          未加密的密码
             * @param  {Function} callback          登录成功的回调函数
             * @param  {[type]}   DontEncodePassword 是否需要加密密码
             * @return {[type]}                     [description]
             */
            login: function(username, password, callback,DontEncodePassword) {
                var $this = this;
                var loginPassword=encodePassword(password);
                if(typeof DontEncodePassword!='undefined'&&DontEncodePassword){
                    loginPassword=password;
                }
                return resource.login(null, param({
                    userName: username,
                    password:loginPassword,
                    type: 2 //1.手机校验码登录，2.账号密码登录
                }), function(respones) {
                    Storage.set(userInfoKey, respones.data);
                    //这里保存用户已登录的状态数据
                    Storage.set(userInfoLogin, {
                        "username": username,
                        "password": loginPassword
                    });
                    $rootScope.globalUserIsLogin = true;
                    Storage.set(loginTypeKey,"phone");
                    $rootScope.loginType="phone";
                    callback && callback(respones);
                })
            },
            /**
             * 注册
             * @param  {[type]} phone      [description]
             * @param  {[type]} verifyCode [description]
             * @param  {[type]} password   [description]
             * @return {[type]}            [description]
             */
            signup: function(phone, verifyCode, password) {
                //这里来对数据进行加密操作
                return resource.signup(null, param({
                    userName: phone,
                    verifyCode: verifyCode,
                    password: encodePassword(password),
                    type: 2
                }));
            },
            /**
             * 退出登录
             * @return {[type]} [description]
             */
            logout: function(callback) {
                Storage.remove(userInfoKey);
                Storage.remove(userInfoLogin);
                $rootScope.globalUserInfo = null;
                $rootScope.globalUserIsLogin = false;
                resource.logout().$promise.then(function(respones) {
                    callback && callback(respones);
                })
            },
            common: function(fun, callback) {
                fun().$promise.then(callback);
            },
            /**
             * 通过登录名获取登录用户信息
             * @return {[type]}           [description]
             */
            get_user_info: function(callback) {
                return resource.get_user_info({}, null, function(respones) {
                    Storage.set(userInfoKey, respones.data);
                    $rootScope.globalUserInfo = respones.data;
                    callback && callback(respones.data);
                })
            },
            /**
             * 获取当前登录用户信息
             * @return {[type]} [description]
             */
            getCurrentUser: function() {
                return Storage.get(userInfoKey);
            },
            /**
             * 判断用户是否登录
             * @return {[type]} [description]
             */
            userIsLogin: function() {
                var userinfo = Storage.get(userInfoKey);
                if (userinfo != null) {
                    if (userinfo.userId != null) {
                        return true;
                    } else {
                        return false;
                    }
                } else {
                    return false;
                }
            },
            /**
             * 修改用户资料信息
             * @param  {[type]} user [description]
             * @return {[type]}      [description]
             */
            update_user_info: function(user, callback) {
                var $this = this;
                user.age=0;
                return resource.update_user_info(null, param(user), function(respones) {
                    //修改用户信息后，重新查询用户信息
                    $this.get_user_info(function(result){
                        callback && callback(result);
                    });
                })
            },
            /**
             * 修改用户密码
             * @param  {[type]} user [description]
             * @return {[type]}      [description]
             */
            update_password: function(account, password, newPwd, callback) {
                return resource.update_password(null, param({
                    userName: account,
                    password: encodePassword(password),
                    newPwd: encodePassword(newPwd),
                    type: 2
                }), function(respones) {
                    //保存最新的账户信息
                    Storage.set('userinfo_login', {
                        username: account,
                        password: encodePassword(newPwd)
                    });
                    callback && callback();
                })
            },
            /**
             * 反馈意见
             * @param  {[type]} content [description]
             * @return {[type]}      [description]
             */
            commit_suggestion: function(content, callback) {
                return resource.commit_suggestion({
                    json: JSON.stringify({
                        content: content
                    })
                }, null, function(respones) {
                    callback && callback();
                })
            },
            getAppId: function() {},
            /**
             * 电信自动登录功能
             * @return {[type]} [description]
             */
            autoLogin: function(scope, isShowSetNickName) {
                var _this = this;
                //获取账号密码
                console.log(DataInterface.getString())
                var userName = JSON.parse(DataInterface.getString()).itvid;
                //识别用户是否登录
                if (userName != null && userName != 'undefined' && userName != 'testott1') { //禁止默认用户
                    var password = 123456;
                    var verifyCode = "-1";
                    userName = userName != "" ? userName : "huawei" + new Date().getTime();
                    /**
                     * 第一步注册
                     * @param  {Object} result) {                               var user [description]
                     * @return {[type]}         [description]
                     */
                    _this.signup("scdx" + userName, verifyCode, password).$promise.then(function(result) {
                        /**
                         * 第二步登录
                         */
                        _this.login("scdx" + userName, password, function() {
                            //获取用户资料
                            _this.get_user_info(function() {
                                //检测用户是否有昵称，如果没有弹出修改菜单
                                if (_this.getCurrentUser().nickName == '') {
                                    scope.user = {};
                                    if (!isShowSetNickName) {
                                        $ionicPopup.show({
                                            title: '设置一个昵称吧',
                                            template: '<input ng-model="user.nickName" type="text" placeholder="请输入用户昵称">',
                                            scope: scope,
                                            buttons: [{
                                                text: '取消'
                                            }, {
                                                text: '确定',
                                                onTap: function(e) {
                                                    $ionicLoading.show();
                                                    var user = _this.getCurrentUser();
                                                    user.nickName = scope.user.nickName;
                                                    _this.update_user_info(user, function() {
                                                        Toast.show('设置成功！');
                                                        $ionicLoading.hide()
                                                    });
                                                }
                                            }]
                                        });
                                    }
                                }
                            });
                        })
                    })
                } else { //处理用户未登录情况
                    this.logout();
                }
            },
            /**
             * 这里来初始化用户相关登录信息
             * @return {[type]} [description]
             */
            init: function() {
                var $this=this;
                //1.判断是否存储了用户账号密码
                    var loginKey=Storage.get(userInfoLogin);
                    if(loginKey!=null){
                        //这里要判断当前的登录方式
                        var loginType=Storage.get(loginTypeKey);
                        $rootScope.loginType=loginType;
                        if(loginType=="third"){ //第三方登录
                            this.thirdLogin(loginKey.username,loginKey.password,function(){
                                $this.get_user_info();
                            })
                        }else{
                            this.login(loginKey.username,loginKey.password,function(){
                                $this.get_user_info();
                            },true);
                        }
                    }else{
                        //不处理。
                    }
                //2.判断是否有用户信息
                var userinfo = this.getCurrentUser();
                if (userinfo != null) {
                    //设置当前用户状态
                    this.get_user_info();
                    $rootScope.globalUserInfo = userinfo;
                    $rootScope.globalUserIsLogin = true;
                } else {
                    //用户未登录
                }
            },
            /**
             * 第三方登录功能
             * @param  {[type]} platform     平台
             * @param  {[type]} access_token 唯一Code值
             * @return {[type]}              [description]
             */
            thirdLogin: function(platform, access_token,successCallback,errorCallabck) {
                var $this=this;
                var promise = resource.thirdLogin(null, param({
                    typeName: platform,
                    code: access_token
                })).$promise;
                promise.then(function(result){
                    if(result.success){
                        //这里去查询用户信息
                        $this.get_user_info();
                        Storage.set(userInfoLogin, {
                            "username": platform,
                            "password": access_token
                        });
                        Storage.set(loginTypeKey,"third"); 
                        $rootScope.loginType="third";
                        $rootScope.globalUserIsLogin = true;
                        successCallback();
                    }else{
                        errorCallabck(result.msg);
                    }
                },function(err){
                    errorCallabck(result.msg);
                })
            }
        }
    })