'use strict';
/**
 * @description 用于给三边页面提供进度状态数据服务，（播放进度、状态等操作）
 * @author superK
 * @link http://www.woaiso.com
 * @date 2015-11-12 10:55:21
 */
rootModule
    .service('VideoStatusCore', function(InterfaceSide, Side, Toast, Movies) {
        var coreData = {}; //用来存储内部使用的相关数据
        var useServerStatus = false;
        var selectMovieId = -1;
        /**
         * 这里的主要功能是用来给三边页面提供视频状态数据服务
         * 目前的数据提供者主要有以下几种
         * 1.TvBox机顶盒提供的状态数据,地址：http://tvbox.handsight.cn
         * 2.四川电信机IPTV机顶盒提供的状态数据，分为a.直接在电视上点播,b.4K超高清甩屏到电视上
         * 3.4K超高清APP播放界面提供的播放进度
         */
        /**
         * 获取状态数据
         * @param  {[type]} options         传入的参数
         * @param  {[type]} successCallback 获取成功执行的回调函数
         * @param  {[type]} errorCallback   失败后的回调函数
         * @return {[type]}                 [description]
         */
        function getStatus(options, successCallback, errorCallback) {
            //这里来判断类型,调用不同的获取方式
            getStatusByPlatform(options, successCallback, errorCallback)
        }
        /**
         * 根据不同的平台获取状态数据
         * @param  {[type]} options         [description]
         * @param  {[type]} successCallback [description]
         * @param  {[type]} errorCallback   [description]
         * @return {[type]}                 [description]
         */
        function getStatusByPlatform(options, successCallback, errorCallback) {
            switch (options.type) {
                case 1: //IPTV获取状态数据
                    getStatusFormIPTV(options, successCallback, errorCallback);
                    break;
                case 2: //系统的平台获取数据
                    getStatusFormSystem(options, successCallback, errorCallback);
                    break;
                case 3: //插件获取进度
                    getStatusFormPlugin(successCallback, errorCallback);
                    break;
                case 4:
                    //写独立app电影拉去播放进度，通过场次id查询电影播放时间，与当前时间比较，返回播放进度
                    getStatusFormMovie(options, successCallback, errorCallback)
                    break;
                default:
                    errorCallback('options.type undefined');
                    break;
            }
        }

        /**
         * 从电影院的某个场次的电影获取状态数据markc
         * @param  {[type]} options         传入的参数，如video ID
         * @param  {[type]} successCallback [description]
         * @param  {[type]} errorCallback   [description]
         * @return {[type]}                 [description]
         */
        function getStatusFormMovie(options, successCallback, errorCallback) {
            if (selectMovieId !=options.showtimeId.videoId) {
                selectMovieId=options.showtimeId.videoId;
                HSData.itvId = "";
                HSData.status = options.showtimeId.status;
                HSData.programtype = 0;
                HSData.programid = options.showtimeId.videoId; //这里使用的是场次Id
                HSData.programname = options.showtimeId.moviename; //电影名字
                HSData.programitem = "";
                HSData.relTime = options.showtimeId.startedTime;
                HSData.movieLength = options.showtimeId.movieLength;
                successCallback(HSData);

                // selectMovieId=options.showtimeId;
                // //通过传入一个电影场次Id 从服务端获取状态数据
                // Movies.getShowSessionInfor(options.showtimeId,function(result) {
                //     if (result.success) {
                //         var data = result.data;
                //         HSData.itvId = "";
                //         HSData.status = data.status;
                //         HSData.programtype = 0;
                //         HSData.programid = data.videoId; //这里使用的是场次Id
                //         HSData.programname = data.moviename; //电影名字
                //         HSData.programitem = "";
                //         HSData.relTime = data.startedTime;
                //         HSData.movieLength = data.movieLength;
                //         successCallback(HSData);
                //     } else {
                //         errorCallback('load status from iptv error' + result.msg);
                //     }
                // }, errorCallback);
            } else {
                HSData.relTime++; //自动做时间++操作
                successCallback(HSData);
            }


        }
        /**
         * 从电视IPTV获取状态数据
         * @param  {[type]} options         传入的参数，如video ID
         * @param  {[type]} successCallback [description]
         * @param  {[type]} errorCallback   [description]
         * @return {[type]}                 [description]
         */
        function getStatusFormIPTV(options, successCallback, errorCallback) {

            //通过传入一个ITVID 从服务端获取状态数据
            InterfaceSide.getDeviceStatus(options.itvId).$promise.then(function(result) {

                if (result.success) {
                    var data = result.data;
                    //这里是从IPTV获取的进度信息
                    //因为在手机甩屏的时候，直接会通过服务端去取状态数据，有可能IPTV平台还没有发送进度信息给
                    //后台服务器，此时去取状态数据，可能会取到上一次的状态数据。
                    //这里对时间进度做一个处理，如果当前的时间和服务器的时间误差很大，则直接将该次的时间抛弃
                    if (!useServerStatus) {
                        if (Math.abs(HSData.relTime - parseInt(data.relTime)) < 30) {
                            useServerStatus = true;
                        } else {
                            HSData.relTime++; //自动做时间++操作
                        }
                    } else {

                    }
                    //判断当前是否使用服务器时间
                    if (useServerStatus) {
                        HSData.itvId = data.itvid;
                        HSData.status = data.status;
                        HSData.programtype = data.programtype;
                        HSData.programid = data.videoId; //这里使用的是VIDEOID
                        HSData.programname = data.programname;
                        HSData.programitem = data.programitem;
                        HSData.relTime = data.relTime;
                    } else {

                    }
                    successCallback(HSData);
                } else {
                    errorCallback('load status from iptv error' + result.msg);
                }
            }, errorCallback);
        }
        /**
         * 独立客户端使用的获取状态数据方法
         * @param  {[type]} options         传入的参数，如video ID
         * @param  {[type]} successCallback [description]
         * @param  {[type]} errorCallback   [description]
         * @return {[type]}                 [description]
         */
        function getStatusFormSystem(options, successCallback, errorCallback) {
            //通过传入一个ITVID 从服务端获取状态数据
            Side.getDeviceStatus(options.deviceId).$promise.then(function(result) {
                if (result.success) {
                    var data = result.data;
                    HSData.itvId = data.itvid;
                    HSData.status = data.status;
                    HSData.programtype = data.programtype;
                    HSData.programid = data.videoId; //独立客户端这里使用的是VIDEOID
                    HSData.programname = data.programname;
                    HSData.programitem = data.programitem;
                    HSData.relTime = data.relTime;
                    HSData.video_start_time = data.get("video_start_time");
                    successCallback(HSData);
                } else {
                    errorCallback('load status from iptv error' + result.msg);
                }
            }, errorCallback);
        }
        /**
         * 存储状态的数据结构
         */
        var HSData = {
            itvId: "",
            status: 3, //当前状态( 0：播放中，1：暂停，2：退出 ，3：进入未开始播放，4：播放完未退出)
            programtype: 0, //0：点播，1:直播，2：录播(回看)
            //内容id，关联内容，连续剧则关联具体集数
            // 点播为：视频ID
            // 直播为：频道ID
            // 录播为：频道ID
            programid: -1, //内容id，关联内容，连续剧则关联具体集数
            programname: "未定义", //当前播放内容的名称（点播为片名，直播为频道名）
            programitem: 0, //第几集(没有集数时，值为0)
            // 点播：从视频起始点到当前播放时间以秒为单位的整数
            // 直播：直接给0
            // 录播：从视频起始点到当前播放时间以秒为单位的整数
            relTime: 0,
            play_screen: 1, //当前在什么屏幕下播放（1:半屏状态，2:全屏状态，3：甩屏到电视上）
            video_start_time: "", //节目开始播放时间,格式：yyyyMMddHHmmss
            videoImgPath: "" //视频快照目录
        }

        /**
         * 从Android、IOS插件获取进度
         * @param  {[type]} successCallback [description]
         * @param  {[type]} errorCallback   [description]
         * @return {[type]}                 [description]
         */
        function getStatusFormPlugin(successCallback, errorCallback) {
            //从插件提供的DataInterface接口获取需要的参数
            if (typeof DataInterface != 'undefined') {
                var dinterface = DataInterface;
                var temp = {}
                temp.itvId = dinterface.get('itvid');
                temp.status = dinterface.get("status");
                temp.programtype = parseInt(dinterface.get("programtype"));
                temp.programid = dinterface.get("programid");
                temp.programname = dinterface.get("programname");
                temp.programitem = dinterface.get("programitem");
                temp.relTime = parseInt(dinterface.get("timeshift"));
                temp.play_screen = dinterface.get("play_screen");
                temp.video_start_time = dinterface.get("video_start_time");


                //1.处理直播和回看功能的时间问题
                //这里识别是否为直播或回看，需要对时间做一个加法
                if (temp.programtype == 1) { //直播
                    //获取当前时间
                    var milliseconds = new Date().getTime();
                    temp.relTime = parseInt(milliseconds / 1000);
                } else if (temp.programtype == "2") { //回看
                    var t = temp.video_start_time;
                    var calcTime = new Date(t.substring(0, 4), parseInt(t.substring(4, 6)) - 1, t.substring(6, 8), t.substring(8, 10), t.substring(10, 12), t.substring(12, 14)).getTime()
                    temp.relTime = parseInt(calcTime / 1000) + parseInt(temp.relTime);
                } else { //点播功能

                }
                //自动记时的功能
                if (coreData.relTime != temp.relTime) { //时间不相等了
                    console.log('status update')
                    coreData = angular.copy(temp); //避免引用传递
                    HSData = temp;
                } else {
                    //时间是相等的
                    //判断当前状态是否在播放
                    if (temp.status == 0) {
                        HSData.relTime++; //时间++
                    } else {
                        //当前为暂停状态不处理
                    }
                }
                successCallback(HSData);
            } else {
                errorCallback('DataInterface in not defined');
            }
        }
        /**
         * 提供给上层调用的接口
         */
        return {
            getStatus: getStatus,
            restore:function(){
                useServerStatus = false;
                selectMovieId = -1;
            }
        }
    })